//FILE: cppclient.cpp
//PURPOSE: a simple C++ program to test libgomd: opens the library, gets and instance of
//         libgomd class, uses the class (calls a member function), then exits
//AUTHOR: Gian Paolo Ghilardi, a.k.a. rejected || jp
//MAIL: (jp80@tin.it)

//TO COMPILE: run the Makefile

#include "libiface.h"
#include <iostream>
#include <stdlib.h>
#include <dlfcn.h>

#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1

using namespace std;

int main( int argc, char **argv)
{
	//if no value was inserted at the command line, we try to get the load of
	//a node, i.e. the 7th.
	int nodeId = 1;
	string remoteGomdIP;
	string remoteGomdPort;

	if(argc>1)
	{
		remoteGomdIP = argv[1];
	}
	else
	{
		cout << "Please enter local IP" << endl;
		cin >> remoteGomdIP;
	}


	if(argc>2)
	{
		remoteGomdPort =  argv[2];
	}
	else
	{
		cout << "Please enter local gomd port" << endl;
		cin >> remoteGomdPort;
	}

	/*
		1)Open the library (libgomd.so)
	*/
	cout << "- GOMD C++ CLIENT EXAMPLE\n" << endl;
	cout << "- This simple app shows how to get infos" << endl;
	cout << "- from a local (or remote) gomd daemon using" << endl;
	cout << "- an handy class included in libgomd library." << endl;
	cout << "- This app uses Linux's dynamic loading APIs" << endl;
	cout << "- to load/unload libgond library as needed.\n" << endl;

	cout << "- Trying to open the library..." << endl;
	void* handle = dlopen("/usr/lib/libgomd.dylib", RTLD_LAZY);
	if (!handle)
	{
		cerr << "- Cannot load library: " << dlerror() << endl;
		return EXIT_FAILURE;
	}


	/*
		2)Load symbols to create/destroy the instances of libgomd classes
	*/
	cout << "- Trying to load symbols from the library..." << endl;
	create_t* create_libgomd = (create_t*) dlsym(handle, "create");
	destroy_t* destroy_libgomd = (destroy_t*) dlsym(handle, "destroy");

	/*
		3)Manage symbols errors (if these symbols are not found,
		  an error will be raised)
	*/
	if (!create_libgomd || !destroy_libgomd)
	{
		cerr << "- Cannot load symbols: " << dlerror() << endl;
		return EXIT_FAILURE;
	}

	/*
		4)All went right: print a message.
	*/
	cout << "- Ok: all symbols were found and loaded successfully!!!" << endl;


	/*
		5)Now we create an instance of the class libgomd
	*/
	libiface* lg = create_libgomd();

	/*
		6)Now we get the load of a specific node
		Please notice the different approach:
		- m_id is included in lgomd.hpp
		- getLoad() is implemented in libgomd.cpp
	*/
	cout << "- LIBGOMD:  this is libgomd instance #" << lg->m_id << endl;
	if(argc > 1)
	{
		//if inserted,the first param of command line is "our" nodeID
		nodeId = atoi(argv[1]);
	}
	//string ret =  lg->getInfoAndCloseConn(remoteGomdIP,atoi(remoteGomdPort.c_str()),"get load 2"); //NOTE: getLoad( nodeID ) prints the value of a nodeId!

	string ret;
	cout << "- LIBGOMD: open connection to: " << remoteGomdIP << endl;
	if(lg->openConn(remoteGomdIP, atoi(remoteGomdPort.c_str()))==true)
	{
		cout << "- LIBGOMD: test #1 (get oM cpus value for all the nodes)" << endl;
		ret = "<ERROR>";
		ret = lg->getInfo("get cpus ALL");
		if(ret=="<ERROR>")
		{
			cout << "- LIBGOMD: cant' get values from remote gomd!" << endl;
			cout << "- Tests _not_ passed!" << endl;
			cout << "- Exiting..." << endl;
			exit(EXIT_FAILURE);
		}
		else
		{
			cout << "- LIBGOMD: value returned by the library: \n  =>" << ret << endl;
			ret = lg->getStringOfInfos(ret,"<-->","\n   >>> ");
			cout << "- LIBGOMD: value parsed via the library: \n   >>> " << ret << endl;
		}

		cout << "- LIBGOMD: reusing connection to: " << remoteGomdIP << endl;

		cout << "- LIBGOMD: test #2 (get oM load value for all the nodes)" << endl;
		ret = "<ERROR>";
		ret = lg->getInfo("get load ALL");
		if(ret=="<ERROR>")
		{
			cout << "- LIBGOMD: cant' get values from remote gomd!" << endl;
			cout << "- Tests _not_ passed!" << endl;
			cout << "- Exiting..." << endl;
			exit(EXIT_FAILURE);
		}
		else
		{
			cout << "- LIBGOMD: value returned by the library: \n  =>" << ret << endl;
			ret = lg->getStringOfInfos(ret,"<-->","\n   >>> ");
			cout << "- LIBGOMD: value parsed via the library: \n   >>> " << ret << endl;
		}

		cout << "- LIBGOMD: closing connection..." << endl;
		lg->closeConn();
		cout << "- Tests succesfully passed." << endl;
		cout << "- Exiting..." << endl;
		return (EXIT_SUCCESS);
	}
	else
	{
		cout << "- <ERROR> Can't contact gomd daemon!" << endl;
		cout << "- <ERROR> Exiting..." << endl;
		return (EXIT_FAILURE);
	}
}
